/**********************************************************************
 *
 * converter.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: converter.h 150 1999-02-05 10:42:50Z rjmcnab $
 *
 *********************************************************************/


#ifndef CONVERTER_H
#define CONVERTER_H


#include "gsdlconf.h"
#include "text_t.h"
#include "gsdlunicode.h"

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace\std\map>
#elif defined(GSDL_USE_STL_H)
#  include <map.h>
#else
#  include <map>
#endif


class converterinfo {
public:
  text_t name;
  inconvertclass *inconverter;
  rzwsoutconvertclass *outconverter;

  converterinfo() {inconverter=NULL; outconverter=NULL;}
  ~converterinfo() {
    if (inconverter!=NULL) {
      delete inconverter;
      inconverter=NULL;
    }
    if (outconverter!=NULL) {
      delete outconverter;
      outconverter=NULL;
    }
  }
};



typedef map<text_t, converterinfo, lttext_t> convertmap;

// convertinfoclass is used to store a number of converters
// indexing them by their name
class convertinfoclass {
protected:
  convertmap converters;

public:
  // type support for convertmap
  typedef convertmap::iterator iterator;
  typedef convertmap::const_iterator const_iterator;
  typedef convertmap::reference reference;
  typedef convertmap::const_reference const_reference;
  typedef convertmap::size_type size_type;
  typedef convertmap::difference_type difference_type;
  typedef convertmap::const_reverse_iterator const_reverse_iterator;
  typedef convertmap::reverse_iterator reverse_iterator;
  
  // constructors
  convertinfoclass () {}

  // basic container support
  iterator begin () {return converters.begin();}
  const_iterator begin () const {return converters.begin();}
  iterator end () {return converters.end();}
  const_iterator end () const {return converters.end();}

  void erase(iterator pos) {converters.erase(pos);}
  void erase(iterator first, iterator last) {converters.erase(first, last);}
  convertinfoclass &operator=(const convertinfoclass &x) 
    {converters=x.converters;return *this;}

  bool empty () const {return converters.empty();}
  size_type size() const {return converters.size();}


  // added functionality
  void clear () {converters.erase(converters.begin(),converters.end());}

  // the converters within converterinfo become the property of
  // of this class after add_converter has been called. They should
  // be allocated with new but not explicitly destroyed (they will
  // be destroyed when this class is destroyed).
  void add_converter (const text_t &name, inconvertclass *inconverter,
		      rzwsoutconvertclass *outconverter);

  // get_inconverter will return NULL if the convert could not be found
  inconvertclass *get_inconverter (const text_t &name);

  // get_outconverter will return NULL if the convert could not be found
  rzwsoutconvertclass *get_outconverter (const text_t &name);
};




#endif
